package jp.co.sds_corp.hibiya.monitor;

import java.util.Date;
import java.util.Vector;

import jp.co.sds_corp.hibiya.connector.IConnector;
import jp.co.sds_corp.hibiya.server.CheckerChangeEvent;
import jp.co.sds_corp.hibiya.server.IChecker;
import jp.co.sds_corp.hibiya.server.ICheckerListener;
import jp.co.sds_corp.util.common.CmnDate;

/**
* グラフのデータを保持します。
*/
public class GraphBuffer extends Vector<GraphBuffer.Data>
	implements ICheckerListener
{
	private static final long serialVersionUID = 1L;
	IConnector con;
	IChecker chk;
	GraphParam prm;
	Vector<Data> vec = new Vector<Data>();
	Vector<ICheckerListener> vecLsn = new Vector<ICheckerListener>();
	CmnDate	last;
	double	fMax = Double.MIN_VALUE;
	double	fMin = Double.MAX_VALUE;

	GraphBuffer( IConnector con, IChecker chk, GraphParam prm ) {
		this.con = con;
		this.chk = chk;
		this.prm = prm;
	}

	/**
	* ICheckerの状態に変更があった時に呼び出されます。
	* @param	ev		状態の変更内容を指定します。
	*/
	public void checkerChanged( CheckerChangeEvent ev ) throws Exception {
		IChecker c = (IChecker )ev.getSource();
		if ( !chk.getName().equals( c.getName() )) {
			return;
		}
		Object res = c.getResult( prm.getValue() );
		Data data = new Data( ev.getWhen(), res );
		add( data );
		last = data.getTime();
		if ( size() > 10000 ) {
			remove( 0 );
		}

		for ( int i = 0; i < vecLsn.size(); i++ ) {
			ICheckerListener l = (ICheckerListener )vecLsn.get( i );
			l.checkerChanged( ev );
		}
	}

	/**
	* 状態の変更を受け取るオブジェクトを追加します。
	* @param	l		状態の変更を受け取るオブジェクト
	*/
	public void addCheckerListener( ICheckerListener l ) {
		vecLsn.add( l );
	}
	/**
	* 状態の変更を受け取るオブジェクトを削除します。
	* @param	l		状態の変更を受け取るオブジェクト
	*/
	public void removeCheckerListener( ICheckerListener l ) {
		vecLsn.remove( l );
	}

	public CmnDate getLastUpdated() {
		return last;
	}

	public class Data
	{
		CmnDate time;
		Object	data;
		Data( Date time, Object data ) {
			this.time = new CmnDate( time );
			this.data = data;
		}
		Data( Object data ) {
			time = new CmnDate();
			this.data = data;
		}
		public CmnDate getTime() {
			return time;
		}
		public Object getData() {
			return data;
		}

	}

}
