package jp.co.sds_corp.hibiya.monitor;

import java.lang.reflect.Method;
import java.text.SimpleDateFormat;
import java.util.Iterator;
import java.util.Map;
import java.util.ResourceBundle;
import java.util.Vector;

import javax.swing.JTable;
import javax.swing.table.AbstractTableModel;

import jp.co.sds_corp.hibiya.connector.IConnector;
import jp.co.sds_corp.hibiya.server.CheckerChangeEvent;
import jp.co.sds_corp.hibiya.server.IChecker;
import jp.co.sds_corp.hibiya.server.ICheckerListener;
import jp.co.sds_corp.util.daemon.core.Interval;

/**
* {@link ListTable}の{@link JTable}のためのデータモデルです。
*/
public class ListModel extends AbstractTableModel
	implements ICheckerListener
{
	private static final long serialVersionUID = 1L;
	Vector<IChecker>	vdat = new Vector<IChecker>();
	MonitorParam param;
	boolean bOrd = true;
	SimpleDateFormat sdf1 = new SimpleDateFormat( "yyyy/MM/dd" );
	SimpleDateFormat sdf2 = new SimpleDateFormat( "HH:mm:ss.SSS" );
	IConnector	connector;
	ResourceBundle	res;
	String[]	units;

	public ListModel( MonitorParam param ) {
		this.param = param;
		res = ResourceBundle.getBundle( "resource.monitor" );
		units = new String[5];
		for ( int i = 0; i < units.length; i++ ) {
			units[i] = res.getString( "unit" + (i+1) + ".text" );
		}
	}

	public void clear() {
		vdat.clear();
	}

	public void add( Object obj ) {
		IChecker chk = (IChecker )obj;
		vdat.add( chk );
		int nIdx = vdat.indexOf( chk );
		fireTableRowsInserted( nIdx, nIdx );
	}

	public void remove( Object obj ) {
		int nIdx = vdat.indexOf( obj );
		if ( nIdx >= 0 ) {
			vdat.remove( obj );
			fireTableRowsDeleted( nIdx, nIdx );
		}
	}

	public void updateChecker() throws Exception {
		vdat.clear();
		Map<String,IChecker> map = connector.getCheckerMap();
		Iterator<String> itr = map.keySet().iterator();
		while ( itr.hasNext()) {
			Object key = itr.next();
			IChecker chk = (IChecker )map.get( key );
			if ( param.getCheckerClass().equals( chk.getClass() ) &&
				chk.getCheckerStatus() != IChecker.Status.DEL )
			{
				add( chk );
			}
		}
		fireTableDataChanged();
	}

	/**
	* ICheckerの状態に変更があった時に呼び出されます。
	* @param    ev       状態の変更内容を指定します。
	*/
	public void checkerChanged( CheckerChangeEvent ev ) throws Exception {
		if ( connector == null || !connector.equals( ev.getConnector())) {
			return;
		}
		IChecker chk = (IChecker )ev.getSource();
		if ( ev.getNewStatus() == IChecker.Status.DEL ) {
			remove( chk );
		} else {
			setData( chk );
		}
	}

	public IConnector getConnector() {
		return this.connector;
	}
	public void setConnector( IConnector var ) {
		this.connector = var;
	}

	public Object getData( int nPos ) {
		return vdat.get( nPos );
	}

	public void setData( IChecker pchk ) {
		for ( int i = 0; i < vdat.size(); i++ ) {
			IChecker chk = (IChecker )vdat.get( i );
			if ( chk.getHashCode() == pchk.getHashCode() ) {
				vdat.remove( i );
				vdat.add( i, pchk );
				fireTableRowsUpdated( i, i );
				break;
			}
		}
	}

	public int getColumnCount() {
		return param.getColumnCount();
	}
	public String getColumnName( int nClm ) {
		return param.getColumnName( nClm );
	}
	public int getRowCount() {
		return vdat.size();
	}

	public Object getValueAt( int row, int col ) {
		IChecker chk = (IChecker )vdat.get( row );
		String name = param.getColumnName( col );
		if ( name.equals( " " )) {
			return Integer.valueOf( row );
		} else if ( param.isMask( name )) {
			return "*";
		} else {
			Class<?>[] cls = {};
			Object[] prm = {};
			try {
				Method mtd = chk.getClass().getMethod( "get" + name, cls );
				Object obj = mtd.invoke( chk, prm );
				if ( obj == null ) {
					return null;
				}
				if ( name.equals( "Interval" )) {
					return intervalToString( (Number )obj );
				}
				return obj.toString();
			} catch ( Exception e ) {
				return "(unknown)";
			}
		}
	}

	public String intervalToString( Number var ) {
		Interval.Type type = Interval.parseType( var.longValue() );
		long unit = Interval.parseUnit( var.longValue() );
		String s = "";
		switch ( type ) {
		case MILLISEC: s = units[0]; break;
		case SEC: s = units[1]; break;
		case MIN: s = units[2]; break;
		case HOUR: s = units[3]; break;
		case DATE: s = units[4]; break;
		}
		return "" + unit + s;
	}

	public boolean isCellEditable( int row, int col ) {
		return false;
	}
}

