package jp.co.sds_corp.hibiya.monitor;

import java.awt.BorderLayout;
import java.awt.Font;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.KeyEvent;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;

import javax.swing.ButtonGroup;
import javax.swing.JCheckBoxMenuItem;
import javax.swing.JFrame;
import javax.swing.JMenu;
import javax.swing.JMenuBar;
import javax.swing.JMenuItem;
import javax.swing.JOptionPane;
import javax.swing.JRadioButtonMenuItem;
import javax.swing.SwingConstants;
import javax.swing.UIManager;
import javax.swing.WindowConstants;

import jp.co.sds_corp.util.common.CmnProperty;
import jp.co.sds_corp.util.config.IConfiguration;
import jp.co.sds_corp.util.window.StatusBar;

/**
* Hibiya Monitorのモニタのメインダイアログです。
*/
public class MonitorDialog extends JFrame
{
	private static final long serialVersionUID = 1L;
	MonitorPanel	panel;
	JMenuBar		menuBar;
	StatusBar		stat;
	CmnProperty		res;

	public MonitorDialog() throws Exception {
		super();
		res = new CmnProperty( "resource.monitor" );
		createControls();
	}

	public void init( IConfiguration config ) throws Exception {
		panel.init( config );
		setBounds( panel.getX(), panel.getY(),
					panel.getWidth(), panel.getHeight() );
	}

	protected void createControls() throws Exception {
		getContentPane().setLayout( new BorderLayout() );
		setTitle( "Hibiya Monitor" );

		panel = new MonitorPanel();
		getContentPane().add( panel );
		setMenu();
		addWindowListener( new WindowAdapter() {
			public void windowClosing( WindowEvent ev ) {
				closeWindow();
			}
		} );
		setDefaultCloseOperation( WindowConstants.DO_NOTHING_ON_CLOSE );

		Runtime rt = Runtime.getRuntime();
		rt.addShutdownHook( new Thread() {
			public void run() {
				panel.stop();
			}
		} );
		stat = new StatusBar( 2 );
		stat.setWidth( 0, 100 );
		stat.setHorizontalAlignment( 1, SwingConstants.RIGHT );
		stat.setText( 0, " " );
		getContentPane().add( stat, BorderLayout.SOUTH );
	}

	private void closeWindow() {
		String msg = res.getString( "window.close.message" );
		String title = res.getString( "window.close.title" );
		int nRet = JOptionPane.showConfirmDialog( this,
					msg, title, JOptionPane.YES_NO_OPTION );
		if ( nRet == JOptionPane.YES_OPTION ) {
			setVisible( false );
			System.exit( 0 );
		}
	}

	// メニューの作成
	public void setMenu() {
		JMenu		menu, submenu;
		JMenuItem	menuItem;
		JCheckBoxMenuItem		cbMenuItem;
		JRadioButtonMenuItem	rbMenuItem;
		ButtonGroup group;

		String text;
		menuBar = new JMenuBar();

		// ファイル
		text = res.getString( "menu1.text", null );
		if ( text != null ) {
			menu = new JMenu( text );
			menu.setMnemonic( KeyEvent.VK_F );
			menuBar.add( menu );

			text = res.getString( "menu1.item1.text", null );
			if ( text != null ) {
				menuItem = new JMenuItem( text, KeyEvent.VK_S );
				menuItem.addActionListener( new ActionListener() {
					public void actionPerformed( ActionEvent ev ) {
						panel.setOption();
					}
				});
				menu.add( menuItem );
			}
			text = res.getString( "menu1.item2.text", null );
			if ( text != null ) {
				menuItem = new JMenuItem( text, KeyEvent.VK_A );
				menuItem.addActionListener( new ActionListener() {
					public void actionPerformed( ActionEvent ev ) {
						panel.saveConfig();
					}
				});
				menu.add( menuItem );
			}
			menu.addSeparator();
			text = res.getString( "menu1.item3.text", null );
			if ( text != null ) {
				menuItem = new JMenuItem( text, KeyEvent.VK_X );
				menuItem.addActionListener( new ActionListener() {
					public void actionPerformed( ActionEvent ev ) {
						closeWindow();
					}
				});
				menu.add( menuItem );
			}
		}

		// 編集
		text = res.getString( "menu2.text", null );
		if ( text != null ) {
			menu = new JMenu( text );
			menu.setMnemonic( KeyEvent.VK_E );
			menuBar.add( menu );

			text = res.getString( "menu2.item1.text", null );
			if ( text != null ) {
				menuItem = new JMenuItem( text, KeyEvent.VK_S );
				menuItem.addActionListener( new ActionListener() {
					public void actionPerformed( ActionEvent ev ) {
						panel.setChecker();
					}
				});
				menu.add( menuItem );
			}
			text = res.getString( "menu2.item2.text", null );
			if ( text != null ) {
				menuItem = new JMenuItem( text, KeyEvent.VK_C );
				menuItem.addActionListener( new ActionListener() {
					public void actionPerformed( ActionEvent ev ) {
						panel.setColumn();
					}
				});
				menu.add( menuItem );
			}
			menu.addSeparator();
			text = res.getString( "menu2.item3.text", null );
			if ( text != null ) {
				menuItem = new JMenuItem( text, KeyEvent.VK_A );
				menuItem.addActionListener( new ActionListener() {
					public void actionPerformed( ActionEvent ev ) {
						panel.addChecker();
					}
				});
				menu.add( menuItem );
			}
			text = res.getString( "menu2.item4.text", null );
			if ( text != null ) {
				menuItem = new JMenuItem( text, KeyEvent.VK_R );
				menuItem.addActionListener( new ActionListener() {
					public void actionPerformed( ActionEvent ev ) {
						panel.removeChecker();
					}
				});
				menu.add( menuItem );
			}
			text = res.getString( "menu2.item5.text", null );
			if ( text != null ) {
				menu.addSeparator();
				submenu = new JMenu( text );
				submenu.setMnemonic( KeyEvent.VK_O );
				menu.add( submenu );
				text = res.getString( "menu2.item5.item1.text", null );
				if ( text != null ) {
					menuItem = new JMenuItem( text, KeyEvent.VK_A );
					menuItem.addActionListener( new ActionListener() {
						public void actionPerformed( ActionEvent ev ) {
							panel.addConnector();
						}
					});
					submenu.add( menuItem );
				}
				text = res.getString( "menu2.item5.item2.text", null );
				if ( text != null ) {
					menuItem = new JMenuItem( text, KeyEvent.VK_3 );
					menuItem.addActionListener( new ActionListener() {
						public void actionPerformed( ActionEvent ev ) {
							panel.editConnector();
						}
					});
					submenu.add( menuItem );
				}
				text = res.getString( "menu2.item5.item3.text", null );
				if ( text != null ) {
					menuItem = new JMenuItem( text, KeyEvent.VK_R );
					menuItem.addActionListener( new ActionListener() {
						public void actionPerformed( ActionEvent ev ) {
							panel.removeConnector();
						}
					});
					submenu.add( menuItem );
				}
			}
		}

		// 表示
		text = res.getString( "menu3.text", null );
		if ( text != null ) {
			menu = new JMenu( text );
			menu.setMnemonic( KeyEvent.VK_D );
			menuBar.add( menu );
			text = res.getString( "menu3.item1.text", null );
			if ( text != null ) {
				menuItem = new JMenuItem( text, KeyEvent.VK_F );
				menuItem.addActionListener( new ActionListener() {
					public void actionPerformed( ActionEvent ev ) {
						Font f = panel.setFont();
						if ( f != null ) {
							setFont( f );
						}
					}
				});
				menu.add( menuItem );
			}
			text = res.getString( "menu3.item2.text", null );
			if ( text != null ) {
				JMenu subMenu = new JMenu( text );
				subMenu.setMnemonic( KeyEvent.VK_L );
				menu.add( subMenu );
				UIManager.LookAndFeelInfo[] lf = UIManager.getInstalledLookAndFeels();
				for ( int i = 0; i < lf.length; i++ ) {
					menuItem = new JMenuItem( lf[i].getName() );
					menuItem.addActionListener( new ActionListener() {
						public void actionPerformed( ActionEvent ev ) {
							panel.setLF( ev );
						}
					});
					menuItem.setActionCommand( lf[i].getClassName() );
					subMenu.add( menuItem );
				}
			}
			menu.addSeparator();
			text = res.getString( "menu3.item3.text", null );
			if ( text != null ) {
				menuItem = new JMenuItem( text, KeyEvent.VK_H );
				menuItem.addActionListener( new ActionListener() {
					public void actionPerformed( ActionEvent ev ) {
						panel.showHistory();
					}
				});
				menu.add( menuItem );
			}
		}

		// ヘルプ
		text = res.getString( "menu4.text", null );
		if ( text != null ) {
			menu = new JMenu( text );
			menu.setMnemonic( KeyEvent.VK_H );
			menuBar.add( menu );
			text = res.getString( "menu4.item1.text", null );
			if ( text != null ) {
				menuItem = new JMenuItem( text, KeyEvent.VK_G );
				menuItem.addActionListener( new ActionListener() {
					public void actionPerformed( ActionEvent ev ) {
						System.gc();
					}
				});
				menu.add( menuItem );
			}
			text = res.getString( "menu4.item2.text", null );
			if ( text != null ) {
				menuItem = new JMenuItem( text, KeyEvent.VK_A );
				menuItem.addActionListener( new ActionListener() {
					public void actionPerformed( ActionEvent ev ) {
						panel.showHelp();
					}
				});
				menu.add( menuItem );
			}
		}
		setJMenuBar( menuBar );
	}

}
