package jp.co.sds_corp.hibiya.monitor;

import java.util.Iterator;
import java.util.List;
import java.util.TreeMap;
import java.util.Vector;

import javax.swing.Icon;

import jp.co.sds_corp.hibiya.monitor.checker.ICheckerPanel;
import jp.co.sds_corp.hibiya.server.IChecker;
import jp.co.sds_corp.util.config.IConfigSavable;
import jp.co.sds_corp.util.config.IConfigurable;
import jp.co.sds_corp.util.config.IConfiguration;

/**
* {@link IChecker}のグループごとの設定を保持します。
*/
public class MonitorParam implements IConfigurable, IConfigSavable
{
	IConfiguration config;
	SetupParam setup;
	String	name;
	String	monitor;
	Class<? extends IChecker>	monCls;
	String	panelName;
	ICheckerPanel	panel;
	Vector<Column>	vecAll;
	Vector<String>	vecName;
	Vector<Column>	vecDisp;
	Vector<String>	vecDispName;
	Vector<GraphParam>	vecGrf;
	int	interval;
	GraphSetupParam	graphSetupParam;

	public MonitorParam( SetupParam setup ) {
		this.setup = setup;
	}

	/**
	* 設定情報で初期化します。
	* @param	config	設定情報
	*/
	public void init( IConfiguration config ) throws Exception {
		this.config = config;
		vecAll = new Vector<Column>();
		vecName = new Vector<String>();
		vecDisp = new Vector<Column>();
		vecDispName = new Vector<String>();
		vecGrf = new Vector<GraphParam>();

		IConfiguration c;
		name = config.getAttributes().get( "name" );
		panelName = config.getAttributes().get( "panel" );
		panel = (ICheckerPanel )Class.forName( panelName ).newInstance();
		init( panel );
		createGraphParam( config );
	}

	private void createGraphParam( IConfiguration config ) throws Exception {
		graphSetupParam = new GraphSetupParam();
		IConfiguration gc = config.getChild( "Graph" );
		if ( gc == null ) {
			return;
		}
		graphSetupParam.init( gc );
		Iterator<IConfiguration> itr = gc.getChilds( "Series" );
		while ( itr.hasNext()) {
			IConfiguration c = (IConfiguration )itr.next();
			GraphParam grf = new GraphParam();
			grf.init( c );
			vecGrf.add( grf );
		}
	}

	private void init( ICheckerPanel pnl ) throws Exception {
		this.monitor = pnl.getCheckerClass().getName();
		this.monCls = pnl.getCheckerClass();

		vecAll.clear();
		vecAll.addAll( pnl.getAllColumn() );
		vecName.clear();
		for ( int i = 0; i < vecAll.size(); i++ ) {
			Column col = (Column )vecAll.get( i );
			vecName.add( col.getName() );
		}
		vecDisp.clear();
		vecDispName.clear();
		Iterator<IConfiguration> itr = config.getChilds( "Column" );
		while ( itr.hasNext()) {
			IConfiguration c = (IConfiguration )itr.next();
			String name = c.getAttributes().get( "name" );
			Integer width = Integer.valueOf( c.getAttributes().get( "width" ));
			Column col = getColumn( name );
			vecDispName.add( name );
			vecDisp.add( col );
			col.setWidth( width.intValue() );
		}
		if ( vecDisp.size() == 0 ) {
			vecDisp.addAll( vecAll );
			vecDispName.addAll( vecName );
		}
	}

	/**
	* グループのの名前を返します。
	*/
	public String getName() {
		return name;
	}

	/**
	* {@link IChecker}のクラス名を返します。
	*/
	public String getChecker() {
		return monitor;
	}
	/**
	* {@link IChecker}のクラスを返します。
	*/
	public Class<? extends IChecker> getCheckerClass() {
		return monCls;
	}

	/**
	* このグループに所属する{@link IChecker}であるかどうかを返します。
	*/
	public boolean isValidChecker( IChecker mon ) {
		if ( monCls.equals( mon.getClass() ) ) {
			return true;
		}
		return false;
	}

	/**
	* {@link IChecker}の設定を行う{@link ICheckerPanel}を返します。
	*/
	public String getPanelName() {
		return panelName;
	}
	/**
	* {@link IChecker}の設定を行う{@link ICheckerPanel}を返します。
	*/
	public ICheckerPanel getPanel() {
		return panel;
	}

	/**
	* 状態別のアイコンを返します。
	* @param	stat	状態
	*/
	public Icon getSizedIcon( IChecker.Status stat ) {
		return setup.getSizedIcon( stat );
	}
	/**
	* 状態別のアイコンを返します。
	* @param	stat	状態
	* @param	nSize	アイコンのサイズ
	*/
	public Icon getSizedIcon( IChecker.Status stat, int nSize ) {
		return setup.getSizedIcon( stat, nSize );
	}

	/**
	* 監視間隔(interval)を設定します。
	* @param	var		設定するデータ
	*/
	public void setInterval( int var ) {
		this.interval = var ;
	}
	/**
	* 監視間隔(interval)を取得します。
	* @return	intervalの内容
	*/
	public int getInterval() {
		return interval;
	}

	/**
	* このグループで表示するカラムを設定します。
	* @param	list	表示するカラム名の入ったList<String>
	*/
	public void setColumn( List<String> list ) {
		vecDisp.clear();
		vecDispName.clear();
		vecDispName.addAll( list );
		for ( int i = 0; i < list.size(); i++ ) {
			String name = (String )list.get( i );
			Column col = getColumn( name );
			vecDisp.add( col );
		}
	}

	/**
	* 表示するカラムの一覧を返します。
	* @return	表示するカラムの一覧List<String>
	*/
	public List<Column> listColumn() {
		return vecDisp;
	}
	/**
	* 表示するカラムの一覧を返します。
	* @return	表示するカラムの一覧List<Column>
	*/
	public List<String> listColumnName() {
		return vecDispName;
	}

	/**
	* 表示するカラム数を返します。
	*/
	public int getColumnCount() {
		return vecDisp.size();
	}

	/**
	* カラムの情報を取得します。
	* @param	name	カラム名
	*/
	public Column getColumn( String name ) {
		int pos = vecName.indexOf( name );
		if ( pos < 0 ) {
			throw( new IllegalArgumentException( "Column not found [" + name + "]" ));
		}
		Column col = (Column )vecAll.get( pos );
		return col;
	}

	/**
	* カラム名を返します。
	* @param	nCol	カラム位置(0〜)
	*/
	public String getColumnName( int nCol ) {
		return (String )vecDispName.get( nCol );
	}

	/**
	* 指定された名前のカラムの位置を返します。
	* @param	name	カラム名
	* @return	カラム位置
	*/
	public int getColumnPos( String name ) {
		return vecDispName.indexOf( name );
	}

	/**
	* このグループの全てのカラムを返します。
	* @return	全てのカラムの一覧List<{@link Column}>
	*/
	public List<Column> listAllColumn() {
		return vecAll;
	}
	/**
	* このグループの全てのカラム名を返します。
	* @return	全てのカラム名一覧List<String>
	*/
	public List<String> listAllName() {
		return vecName;
	}

	/**
	* このグループのグラフの設定を返します。
	* @return	グラフの設定一覧List<GraphParam>
	*/
	public List<GraphParam> listGraphParam() {
		return vecGrf;
	}

	/**
	* カラムの幅を設定します。
	* @param	pos		カラム位置
	* @param	width	カラムの幅
	*/
	public void setColumnWidth( int pos, int width ) {
		Column col = (Column )vecDisp.get( pos );
		col.setWidth( width );
	}

	/**
	* カラムの幅を取得します。
	* @param	pos		カラム位置
	* @return	カラムの幅
	*/
	public int getColumnWidth( int pos ) {
		Column col = (Column )vecDisp.get( pos );
		return col.getWidth();
	}

	/**
	* カラムの順番を設定します。
	* @param	pos		カラム位置
	* @param	order	カラムの順番
	*/
	public void setOrder( int pos, int order ) {
		Column col = (Column )vecDisp.get( pos );
		col.setOrder( order );
	}
	/**
	* カラムの順番を取得します。
	* @param	pos		カラム位置
	* @return	カラムの順番
	*/
	public int getOrder( int pos ) {
		Column col = (Column )vecDisp.get( pos );
		return col.getOrder();
	}

	/**
	* カラムの表示フラグを取得します。
	* @param	name	カラム名
	* @return	表示フラグ
	*/
	public boolean isMask( String name ) {
		int idx = vecName.indexOf( name );
		Column col = (Column )vecAll.get( idx );
		return col.isMask();
	}

	public GraphSetupParam getGraphSetupParam() {
		return graphSetupParam;
	}

	/**
	* 設定情報を保存します。
	* @param	config	設定情報
	*/
	public void saveConfig( IConfiguration config ) {
		config.setAttribute( "name", getName() );
		config.setAttribute( "panel", getPanelName() );

		IConfiguration c;
		TreeMap<Integer, Column> map = new TreeMap<Integer, Column>();
		for ( int i = 0; i < vecDisp.size(); i++ ) {
			Column col = (Column )vecDisp.get( i );
			map.put( Integer.valueOf( col.getOrder()), col );
		}
		Iterator<Integer> itr = map.keySet().iterator();
		while ( itr.hasNext()) {
			Object key = itr.next();
			Column col = (Column )map.get( key );
			c = config.addChild( "Column" );
			c.setAttribute( "name", col.getName() );
			c.setAttribute( "width", String.valueOf( col.getWidth() ));
		}
		if ( vecGrf.size() > 0 ) {
			c = config.addChild( "Graph" );
			graphSetupParam.saveConfig( c );
			for ( int i = 0; i < vecGrf.size(); i++ ) {
				IConfiguration c2 = c.addChild( "Series" );
				GraphParam grf = (GraphParam )vecGrf.get( i );
				grf.saveConfig( c2 );
			}
		}
	}

}
